@@ -54,16 +54,11 @@ fmi3Discrete       = 4
 fmi3Dependent      = 5
 
 # callback functions
-fmi3CallbackLogMessageTYPE         = CFUNCTYPE(None,       fmi3InstanceEnvironment, fmi3String, fmi3Status, fmi3String, fmi3String)
-fmi3CallbackAllocateMemoryTYPE     = CFUNCTYPE(c_void_p,   fmi3InstanceEnvironment, c_size_t, c_size_t)
-fmi3CallbackFreeMemoryTYPE         = CFUNCTYPE(None,       fmi3InstanceEnvironment, c_void_p)
-fmi3CallbackIntermediateUpdateTYPE = CFUNCTYPE(fmi3Status, fmi3InstanceEnvironment, fmi3Float64, fmi3Boolean, fmi3Boolean, fmi3Boolean, fmi3Boolean, fmi3Boolean, fmi3Boolean)
+fmi3CallbackLogMessageTYPE         = CFUNCTYPE(None, fmi3InstanceEnvironment, fmi3String, fmi3Status, fmi3String, fmi3String)
+fmi3CallbackIntermediateUpdateTYPE = CFUNCTYPE(None, fmi3InstanceEnvironment, fmi3Float64, fmi3Boolean, fmi3Boolean, fmi3Boolean, fmi3Boolean, fmi3Boolean, fmi3Boolean, POINTER(fmi3Boolean), POINTER(fmi3Float64))
 fmi3CallbackLockPreemptionTYPE     = CFUNCTYPE(None)
 fmi3CallbackUnlockPreemptionTYPE   = CFUNCTYPE(None)
 
-# allocated memory
-_mem_addr = set()
-
 
 def intermediateUpdate(*args):
     return fmi3OK
@@ -76,24 +71,6 @@ def printLogMessage(instanceEnvironment, instanceName, status, category, message
     print("[%s] %s" % (label, message))
 
 
-def allocateMemory(instanceEnvironment, nobj, size):
-    mem = calloc(nobj, size)
-    _mem_addr.add(mem)
-    return mem
-
-
-def freeMemory(instanceEnvironment, obj):
-
-    if obj is None:
-        return  # NULL pointer
-
-    if obj in _mem_addr:
-        free(obj)
-        _mem_addr.remove(obj)
-    else:
-        print("freeMemory() was called for a pointer (%s) that was not allocated" % obj)
-
-
 def stepFinished(instanceEnvironment, status):
     pass
 
@@ -122,7 +99,7 @@ class _FMU3(_FMU):
         self._fmi3Function('fmi3FreeInstance', [(fmi3Instance, 'instance')], None)
 
         # Enter and exit initialization mode, terminate and reset
-        self._fmi3Function('fmi3SetupExperiment', [
+        self._fmi3Function('fmi3EnterInitializationMode', [
             (fmi3Instance, 'instance'),
             (fmi3Boolean,  'toleranceDefined'),
             (fmi3Float64,  'tolerance'),
@@ -131,13 +108,10 @@ class _FMU3(_FMU):
             (fmi3Float64,  'stopTime')
         ])
 
-        self._fmi3Function('fmi3EnterInitializationMode', [(fmi3Instance, 'instance')])
-
         self._fmi3Function('fmi3ExitInitializationMode', [(fmi3Instance, 'instance')])
 
         self._fmi3Function('fmi3EnterEventMode', [
             (fmi3Instance,       'instance'),
-            (fmi3Boolean,        'inputEvent'),
             (fmi3Boolean,        'stepEvent'),
             (POINTER(fmi3Int32), 'rootsFound'),
             (c_size_t,           'nEventIndicators'),
@@ -167,11 +141,11 @@ class _FMU3(_FMU):
         for name, _type in types:
 
             params = [
-                (fmi3Instance, 'instance'),
+                (fmi3Instance,                'instance'),
                 (POINTER(fmi3ValueReference), 'vr'),
-                (c_size_t, 'nvr'),
-                (POINTER(_type), 'value'),
-                (c_size_t, 'nValues')
+                (c_size_t,                    'nvr'),
+                (POINTER(_type),              'value'),
+                (c_size_t,                    'nValues')
             ]
 
             self._fmi3Function('fmi3Get' + name, params)
@@ -242,12 +216,14 @@ class _FMU3(_FMU):
         # Getting partial derivatives
         self._fmi3Function('fmi3GetDirectionalDerivative', [
             (fmi3Instance,                'instance'),
-            (POINTER(fmi3ValueReference), 'vUnknown_ref'),
-            (c_size_t,                    'nUnknown'),
-            (POINTER(fmi3ValueReference), 'vKnown_ref'),
-            (c_size_t,                    'nKnown'),
-            (POINTER(fmi3Float64),        'dvKnown'),
-            (POINTER(fmi3Float64),        'dvUnknown')
+            (POINTER(fmi3ValueReference), 'unknowns'),
+            (c_size_t,                    'nUnknowns'),
+            (POINTER(fmi3ValueReference), 'knowns'),
+            (c_size_t,                    'nKnowns'),
+            (POINTER(fmi3Float64),        'seed'),
+            (c_size_t,                    'nSeed'),
+            (POINTER(fmi3Float64),        'sensitivity'),
+            (c_size_t,                    'nSensitivity')
         ])
 
         # Entering and exiting the Configuration or Reconfiguration Mode
@@ -260,7 +236,8 @@ class _FMU3(_FMU):
             (fmi3Instance,                'instance'),
             (POINTER(fmi3ValueReference), 'valueReferences'),
             (c_size_t,                    'nValueReferences'),
-            (POINTER(fmi3Clock),          'value')
+            (POINTER(fmi3Clock),          'values'),
+            (c_size_t,                    'nValues')
         ])
 
         self._fmi3Function('fmi3SetClock', [
@@ -268,7 +245,8 @@ class _FMU3(_FMU):
             (POINTER(fmi3ValueReference), 'valueReferences'),
             (c_size_t,                    'nValueReferences'),
             (POINTER(fmi3Clock),          'value'),
-            (POINTER(fmi3Boolean),        'subactive')
+            (POINTER(fmi3Boolean),        'subactive'),
+            (c_size_t,                    'nValues')
         ])
 
         self._fmi3Function('fmi3GetIntervalDecimal', [
@@ -283,14 +261,16 @@ class _FMU3(_FMU):
             (POINTER(fmi3ValueReference), 'valueReferences'),
             (c_size_t,                    'nValueReferences'),
             (POINTER(fmi3UInt64),         'intervalCounter'),
-            (POINTER(fmi3UInt64),         'resolution')
+            (POINTER(fmi3UInt64),         'resolution'),
+            (c_size_t,                    'nValues')
         ])
 
         self._fmi3Function('fmi3SetIntervalDecimal', [
             (fmi3Instance,                'instance'),
             (POINTER(fmi3ValueReference), 'valueReferences'),
             (c_size_t,                    'nValueReferences'),
-            (POINTER(fmi3Float64),        'interval')
+            (POINTER(fmi3Float64),        'interval'),
+            (c_size_t,                    'nValues')
         ])
 
         self._fmi3Function('fmi3SetIntervalFraction', [
@@ -298,7 +278,8 @@ class _FMU3(_FMU):
             (POINTER(fmi3ValueReference), 'valueReferences'),
             (c_size_t,                    'nValueReferences'),
             (POINTER(fmi3UInt64),         'intervalCounter'),
-            (POINTER(fmi3UInt64),         'resolution')
+            (POINTER(fmi3UInt64),         'resolution'),
+            (c_size_t,                    'nValues')
         ])
 
         self._fmi3Function('fmi3NewDiscreteStates', [
@@ -322,7 +303,12 @@ class _FMU3(_FMU):
         """
 
         if not hasattr(self.dll, fname):
-            setattr(self, fname, None)
+
+            def raise_exception(*args):
+                raise Exception("Function %s is missing in shared library." % fname)
+
+            setattr(self, fname, raise_exception)
+
             return
 
         if len(params) > 0:
@@ -373,7 +359,7 @@ class _FMU3(_FMU):
 
     # Enter and exit initialization mode, terminate and reset
 
-    def setupExperiment(self, tolerance=None, startTime=0.0, stopTime=None):
+    def enterInitializationMode(self, tolerance=None, startTime=0.0, stopTime=None):
 
         toleranceDefined = fmi3True if tolerance is not None else fmi3False
 
@@ -385,10 +371,7 @@ class _FMU3(_FMU):
         if stopTime is None:
             stopTime = 0.0
 
-        return self.fmi3SetupExperiment(self.component, toleranceDefined, tolerance, startTime, stopTimeDefined, stopTime)
-
-    def enterInitializationMode(self):
-        return self.fmi3EnterInitializationMode(self.component)
+        return self.fmi3EnterInitializationMode(self.component, toleranceDefined, tolerance, startTime, stopTimeDefined, stopTime)
 
     def exitInitializationMode(self):
         return self.fmi3ExitInitializationMode(self.component)
@@ -401,6 +384,14 @@ class _FMU3(_FMU):
 
     # Getting and setting variable values
 
+    def getFloat32(self, vr, nValues=None):
+        if nValues is None:
+            nValues = len(vr)
+        vr = (fmi3ValueReference * len(vr))(*vr)
+        values = (fmi3Float32 * nValues)()
+        self.fmi3GetFloat32(self.component, vr, len(vr), values, nValues)
+        return list(values)
+
     def getFloat64(self, vr, nValues=None):
         if nValues is None:
             nValues = len(vr)
@@ -409,6 +400,38 @@ class _FMU3(_FMU):
         self.fmi3GetFloat64(self.component, vr, len(vr), values, nValues)
         return list(values)
 
+    def getInt8(self, vr, nValues=None):
+        if nValues is None:
+            nValues = len(vr)
+        vr = (fmi3ValueReference * len(vr))(*vr)
+        value = (fmi3Int8 * nValues)()
+        self.fmi3GetInt8(self.component, vr, len(vr), value, nValues)
+        return list(value)
+
+    def getUInt8(self, vr, nValues=None):
+        if nValues is None:
+            nValues = len(vr)
+        vr = (fmi3ValueReference * len(vr))(*vr)
+        value = (fmi3UInt8 * nValues)()
+        self.fmi3GetUInt8(self.component, vr, len(vr), value, nValues)
+        return list(value)
+
+    def getInt16(self, vr, nValues=None):
+        if nValues is None:
+            nValues = len(vr)
+        vr = (fmi3ValueReference * len(vr))(*vr)
+        value = (fmi3Int16 * nValues)()
+        self.fmi3GetInt16(self.component, vr, len(vr), value, nValues)
+        return list(value)
+
+    def getUInt16(self, vr, nValues=None):
+        if nValues is None:
+            nValues = len(vr)
+        vr = (fmi3ValueReference * len(vr))(*vr)
+        value = (fmi3UInt16 * nValues)()
+        self.fmi3GetUInt16(self.component, vr, len(vr), value, nValues)
+        return list(value)
+
     def getInt32(self, vr, nValues=None):
         if nValues is None:
             nValues = len(vr)
@@ -417,6 +440,22 @@ class _FMU3(_FMU):
         self.fmi3GetInt32(self.component, vr, len(vr), value, nValues)
         return list(value)
 
+    def getUInt32(self, vr, nValues=None):
+        if nValues is None:
+            nValues = len(vr)
+        vr = (fmi3ValueReference * len(vr))(*vr)
+        value = (fmi3UInt32 * nValues)()
+        self.fmi3GetUInt32(self.component, vr, len(vr), value, nValues)
+        return list(value)
+
+    def getInt64(self, vr, nValues=None):
+        if nValues is None:
+            nValues = len(vr)
+        vr = (fmi3ValueReference * len(vr))(*vr)
+        value = (fmi3Int64 * nValues)()
+        self.fmi3GetInt64(self.component, vr, len(vr), value, nValues)
+        return list(value)
+
     def getUInt64(self, vr, nValues=None):
         if nValues is None:
             nValues = len(vr)
@@ -439,12 +478,6 @@ class _FMU3(_FMU):
         self.fmi3GetString(self.component, vr, len(vr), value)
         return list(value)
 
-    def getString(self, vr):
-        vr = (fmi3ValueReference * len(vr))(*vr)
-        value = (fmi3String * len(vr))()
-        self.fmi3GetString(self.component, vr, len(vr), value)
-        return list(value)
-
     def getBinary(self, vr):
         vr = (fmi3ValueReference * len(vr))(*vr)
         value = (fmi3Binary * len(vr))()
@@ -452,16 +485,56 @@ class _FMU3(_FMU):
         self.fmi3GetBinary(self.component, vr, len(vr), size, value, len(value))
         return list(value)
 
+    def setFloat32(self, vr, values):
+        vr = (fmi3ValueReference * len(vr))(*vr)
+        values = (fmi3Float32 * len(values))(*values)
+        self.fmi3SetFloat32(self.component, vr, len(vr), values, len(values))
+
     def setFloat64(self, vr, values):
         vr = (fmi3ValueReference * len(vr))(*vr)
         values = (fmi3Float64 * len(values))(*values)
         self.fmi3SetFloat64(self.component, vr, len(vr), values, len(values))
 
+    def setInt8(self, vr, values):
+        vr = (fmi3ValueReference * len(vr))(*vr)
+        values = (fmi3Int8 * len(values))(*values)
+        self.fmi3SetInt8(self.component, vr, len(vr), values, len(values))
+
+    def setUInt8(self, vr, values):
+        vr = (fmi3ValueReference * len(vr))(*vr)
+        values = (fmi3UInt8 * len(values))(*values)
+        self.fmi3SetUInt8(self.component, vr, len(vr), values, len(values))
+
+    def setInt16(self, vr, values):
+        vr = (fmi3ValueReference * len(vr))(*vr)
+        values = (fmi3Int16 * len(values))(*values)
+        self.fmi3SetInt16(self.component, vr, len(vr), values, len(values))
+
+    def setUInt16(self, vr, values):
+        vr = (fmi3ValueReference * len(vr))(*vr)
+        values = (fmi3UInt16 * len(values))(*values)
+        self.fmi3SetUInt16(self.component, vr, len(vr), values, len(values))
+
     def setInt32(self, vr, values):
         vr = (fmi3ValueReference * len(vr))(*vr)
         values = (fmi3Int32 * len(values))(*values)
         self.fmi3SetInt32(self.component, vr, len(vr), values, len(values))
 
+    def setUInt32(self, vr, values):
+        vr = (fmi3ValueReference * len(vr))(*vr)
+        values = (fmi3UInt32 * len(values))(*values)
+        self.fmi3SetUInt32(self.component, vr, len(vr), values, len(values))
+
+    def setInt64(self, vr, values):
+        vr = (fmi3ValueReference * len(vr))(*vr)
+        values = (fmi3Int64 * len(values))(*values)
+        self.fmi3SetInt64(self.component, vr, len(vr), values, len(values))
+
+    def setUInt64(self, vr, values):
+        vr = (fmi3ValueReference * len(vr))(*vr)
+        values = (fmi3UInt64 * len(values))(*values)
+        self.fmi3SetUInt64(self.component, vr, len(vr), values, len(values))
+
     def setBoolean(self, vr, values):
         vr = (fmi3ValueReference * len(vr))(*vr)
         values = (fmi3Boolean * len(values))(*values)
@@ -557,9 +630,7 @@ class FMU3Model(_FMU3):
             (fmi3Boolean,                        'visible'),
             (fmi3Boolean,                        'loggingOn'),
             (fmi3InstanceEnvironment,            'instanceEnvironment'),
-            (fmi3CallbackLogMessageTYPE,         'logMessage'),
-            (fmi3CallbackAllocateMemoryTYPE,     'allocateMemory'),
-            (fmi3CallbackFreeMemoryTYPE,         'freeMemory')
+            (fmi3CallbackLogMessageTYPE,         'logMessage')
         ], fmi3Instance)
 
         self._fmi3Function('fmi3EnterContinuousTimeMode', [(fmi3Instance, 'instance')])
@@ -578,14 +649,14 @@ class FMU3Model(_FMU3):
 
         self._fmi3Function('fmi3SetContinuousStates', [
             (fmi3Instance,         'instance'),
-            (POINTER(fmi3Float64), 'x'),
-            (c_size_t,             'nx')
+            (POINTER(fmi3Float64), 'continuousStates'),
+            (c_size_t,             'nContinuousStates')
         ])
 
         self._fmi3Function('fmi3GetDerivatives', [
             (fmi3Instance,         'instance'),
             (POINTER(fmi3Float64), 'derivatives'),
-            (c_size_t,             'nx')
+            (c_size_t,             'nContinuousStates')
         ])
 
         self._fmi3Function('fmi3GetEventIndicators', [
@@ -596,24 +667,24 @@ class FMU3Model(_FMU3):
 
         self._fmi3Function('fmi3GetContinuousStates', [
             (fmi3Instance,         'instance'),
-            (POINTER(fmi3Float64), 'x'),
-            (c_size_t,             'nx')
+            (POINTER(fmi3Float64), 'continuousStates'),
+            (c_size_t,             'nContinuousStates')
         ])
 
         self._fmi3Function('fmi3GetNominalsOfContinuousStates', [
             (fmi3Instance,         'instance'),
             (POINTER(fmi3Float64), 'nominals'),
-            (c_size_t,             'nx')
+            (c_size_t,             'nContinuousStates')
         ])
 
         self._fmi3Function('fmi3GetNumberOfEventIndicators', [
             (fmi3Instance,      'instance'),
-            (POINTER(c_size_t), 'nz')
+            (POINTER(c_size_t), 'nEventIndicators')
         ])
 
         self._fmi3Function('fmi3GetNumberOfContinuousStates', [
             (fmi3Instance,      'instance'),
-            (POINTER(c_size_t), 'nx')
+            (POINTER(c_size_t), 'nContinuousStates')
         ])
 
     def instantiate(self, visible=False, loggingOn=False):
@@ -622,8 +693,6 @@ class FMU3Model(_FMU3):
 
         # save callbacks from GC
         self.printLogMessage = fmi3CallbackLogMessageTYPE(printLogMessage)
-        self.allocateMemory = fmi3CallbackAllocateMemoryTYPE(allocateMemory)
-        self.freeMemory = fmi3CallbackFreeMemoryTYPE(freeMemory)
 
         self.component = self.fmi3InstantiateModelExchange(
             self.instanceName.encode('utf-8'),
@@ -632,22 +701,19 @@ class FMU3Model(_FMU3):
             fmi3True if visible else fmi3False,
             fmi3True if loggingOn else fmi3False,
             fmi3InstanceEnvironment(),
-            self.printLogMessage,
-            self.allocateMemory,
-            self.freeMemory)
+            self.printLogMessage)
 
         if not self.component:
             raise Exception("Failed to instantiate FMU")
 
     # Enter and exit the different modes
 
-    def enterEventMode(self, inputEvent=False, stepEvent=False, rootsFound=[], timeEvent=False):
+    def enterEventMode(self, stepEvent=False, rootsFound=[], timeEvent=False):
 
         rootsFound = (fmi3Int32 * len(rootsFound))(*rootsFound)
 
         return self.fmi3EnterEventMode(
             self.component,
-            fmi3True if inputEvent else fmi3False,
             fmi3True if stepEvent else fmi3False,
             rootsFound,
             len(rootsFound),
@@ -692,21 +758,21 @@ class FMU3Model(_FMU3):
     def setTime(self, time):
         return self.fmi3SetTime(self.component, time)
 
-    def setContinuousStates(self, x, nx):
-        return self.fmi3SetContinuousStates(self.component, x, nx)
+    def setContinuousStates(self, continuousStates, nContinuousStates):
+        return self.fmi3SetContinuousStates(self.component, continuousStates, nContinuousStates)
 
     # Evaluation of the model equations
 
-    def getDerivatives(self, dx, nx):
-        return self.fmi3GetDerivatives(self.component, dx, nx)
+    def getDerivatives(self, derivatives, nContinuousStates):
+        return self.fmi3GetDerivatives(self.component, derivatives, nContinuousStates)
 
-    def getEventIndicators(self, z, nz):
-        return self.fmi3GetEventIndicators(self.component, z, nz)
+    def getEventIndicators(self, eventIndicators, nEventIndicators):
+        return self.fmi3GetEventIndicators(self.component, eventIndicators, nEventIndicators)
 
-    def getContinuousStates(self, x, nx):
-        return self.fmi3GetContinuousStates(self.component, x, nx)
+    def getContinuousStates(self, continuousStates, nContinuousStates):
+        return self.fmi3GetContinuousStates(self.component, continuousStates, nContinuousStates)
 
-    def getNominalsOfContinuousStatesTYPE(self):
+    def getNominalsOfContinuousState(self):
         pass
 
 
@@ -719,34 +785,23 @@ class FMU3Slave(_FMU3):
 
         super(FMU3Slave, self).__init__(**kwargs)
 
-        self._fmi3Function('fmi3InstantiateBasicCoSimulation', [
+        self._fmi3Function('fmi3InstantiateCoSimulation', [
             (fmi3String,                         'instanceName'),
             (fmi3String,                         'instantiationToken'),
             (fmi3String,                         'resourceLocation'),
             (fmi3Boolean,                        'visible'),
             (fmi3Boolean,                        'loggingOn'),
-            (fmi3Boolean,                        'intermediateVariableGetRequired'),
-            (fmi3Boolean,                        'intermediateInternalVariableGetRequired'),
-            (fmi3Boolean,                        'intermediateVariableSetRequired'),
+            (fmi3Boolean,                        'eventModeRequired'),
+            (POINTER(fmi3ValueReference),        'requiredIntermediateVariables'),
+            (c_size_t,                           'nRequiredIntermediateVariables'),
             (fmi3InstanceEnvironment,            'instanceEnvironment'),
             (fmi3CallbackLogMessageTYPE,         'logMessage'),
-            (fmi3CallbackAllocateMemoryTYPE,     'allocateMemory'),
-            (fmi3CallbackFreeMemoryTYPE,         'freeMemory'),
             (fmi3CallbackIntermediateUpdateTYPE, 'intermediateUpdate')
         ], fmi3Instance)
 
         # Simulating the slave
 
         self._fmi3Function('fmi3EnterStepMode', [(fmi3Instance, 'instance')])
-
-        self._fmi3Function('fmi3SetInputDerivatives', [
-            (fmi3Instance,                'instance'),
-            (POINTER(fmi3ValueReference), 'valueReferences'),
-            (c_size_t,                    'nValueReferences'),
-            (POINTER(fmi3Int32),          'orders'),
-            (POINTER(fmi3Float64),        'values'),
-            (c_size_t,                    'nValues'),
-        ])
         
         self._fmi3Function('fmi3GetOutputDerivatives', [
             (fmi3Instance,                'instance'),
@@ -762,49 +817,36 @@ class FMU3Slave(_FMU3):
             (fmi3Float64,          'currentCommunicationPoint'),
             (fmi3Float64,          'communicationStepSize'),
             (fmi3Boolean,          'noSetFMUStatePriorToCurrentPoint'),
-            (POINTER(fmi3Boolean), 'earlyReturn')
+            (POINTER(fmi3Boolean), 'terminate'),
+            (POINTER(fmi3Boolean), 'earlyReturn'),
+            (POINTER(fmi3Float64), 'lastSuccessfulTime')
         ])
 
         self._fmi3Function('fmi3ActivateModelPartition', [
             (fmi3Instance,       'instance'),
             (fmi3ValueReference, 'clockReference'),
+            (c_size_t,           'clockElementIndex'),
             (fmi3Float64,        'activationTime')
         ])
-        
-        self._fmi3Function('fmi3DoEarlyReturn', [
-            (fmi3Instance, 'instance'),
-            (fmi3Float64,  'earlyReturnTime')
-        ])
 
-        self._fmi3Function('fmi3GetDoStepDiscardedStatus', [
-            (fmi3Instance,         'instance'),
-            (POINTER(fmi3Boolean), 'terminate'),
-            (POINTER(fmi3Float64), 'lastSuccessfulTime')
-        ])
-
-    def instantiate(self, visible=False, loggingOn=False):
+    def instantiate(self, visible=False, loggingOn=False, eventModeRequired=False):
 
         resourceLocation = pathlib.Path(self.unzipDirectory, 'resources').as_uri()
 
         # save callbacks from GC
         self.printLogMessage = fmi3CallbackLogMessageTYPE(printLogMessage)
-        self.allocateMemory = fmi3CallbackAllocateMemoryTYPE(allocateMemory)
-        self.freeMemory = fmi3CallbackFreeMemoryTYPE(freeMemory)
         self.intermediateUpdate = fmi3CallbackIntermediateUpdateTYPE(intermediateUpdate)
 
-        self.component = self.fmi3InstantiateBasicCoSimulation(
+        self.component = self.fmi3InstantiateCoSimulation(
             self.instanceName.encode('utf-8'),
             self.guid.encode('utf-8'),
             resourceLocation.encode('utf-8'),
             fmi3True if visible else fmi3False,
             fmi3True if loggingOn else fmi3False,
-            fmi3False,
-            fmi3False,
-            fmi3False,
+            fmi3True if eventModeRequired else fmi3False,
+            None, 0,
             fmi3InstanceEnvironment(),
             self.printLogMessage,
-            self.allocateMemory,
-            self.freeMemory,
             self.intermediateUpdate)
 
         if not self.component:
@@ -826,9 +868,11 @@ class FMU3Slave(_FMU3):
         return list(value)
 
     def doStep(self, currentCommunicationPoint, communicationStepSize, noSetFMUStatePriorToCurrentPoint=fmi3True):
+        terminate = fmi3Boolean()
         earlyReturn = fmi3Boolean()
-        status = self.fmi3DoStep(self.component, currentCommunicationPoint, communicationStepSize, noSetFMUStatePriorToCurrentPoint, byref(earlyReturn))
-        return status, earlyReturn.value != fmi3False
+        lastSuccessfulTime = fmi3Float64()
+        status = self.fmi3DoStep(self.component, currentCommunicationPoint, communicationStepSize, noSetFMUStatePriorToCurrentPoint, byref(terminate), byref(earlyReturn), byref(lastSuccessfulTime))
+        return status, terminate.value != fmi3False, earlyReturn.value != fmi3False, lastSuccessfulTime.value
 
     def cancelStep(self):
         self.fmi3CancelStep(self.component)
